@@ -35,6 +35,7 @@ PROXY_POOL_URL = ""
 log_style = LOG_STYLE_PRINT
 PROXY_POOL_CAN_RUN_FLAG = True
 
+
 logging.basicConfig(filename="crawler_util.log", level=logging.WARNING)
 
 
@@ -170,9 +171,22 @@ def write_log(string: str = "", print_file: object = sys.stdout):
         logging.error(string)
 
 
-def log(string: str, print_sep: str = ' ', print_end: str = "\n", print_file: object = sys.stdout,
+def log(string: str = "", print_sep: str = ' ', print_end: str = "\n", print_file: object = sys.stdout,
         print_flush: bool = None) -> None:
+    """
+    本项目的通用输出函数， 使用这个方法可以避免tqdm进度条被中断重新输出
+    :param process_bar_file: 如果使用process_bar参数并且需要保持格式不变，则设置此项参数
+    与process_bar的初始化file参数一致，默认为sys.stderr
+    :param string: 待输出字符串
+    :param print_sep: 输出字符串之间的连接符，同print方法sep，目前被废弃
+    :param print_end: 输出字符串之后的结尾添加字符
+    :param print_file: 输出流
+    :param print_flush: 是否强制输出缓冲区，同print方法flush，目前被废弃
+    :return:
+    """
     global log_style
+    print_file = global_val.get_value(KEEP_PROCESS_BAR_STYLE_FILE) \
+        if global_val.get_value(KEEP_PROCESS_BAR_STYLE) else print_file
     string = string if string is not None else ""
     if type(string) != str:
         try:
@@ -182,12 +196,10 @@ def log(string: str, print_sep: str = ' ', print_end: str = "\n", print_file: ob
     if log_style == LOG_STYLE_LOG:
         write_log(string, print_file)
     elif log_style == LOG_STYLE_PRINT:
-        print(get_timestamp() + get_split(lens=3, style=" ")
-              + string, sep=print_sep, end=print_end, file=print_file, flush=print_flush)
+        tqdm.write(s=get_timestamp() + get_split(lens=3, style=" ") + string, file=print_file, end=print_end)
     elif log_style == LOG_STYLE_ALL:
         write_log(string, print_file)
-        print(get_timestamp() + get_split(lens=3, style=" ")
-              + string, sep=print_sep, end=print_end, file=print_file, flush=print_flush)
+        tqdm.write(s=get_timestamp() + get_split(lens=3, style=" ") + string, file=print_file, end=print_end)
 
 
 class CanStopThread(threading.Thread):
@@ -364,8 +376,12 @@ def basic_config(log_file_name: str = "crawler_util.log",
                  enable_server: bool = True,
                  test_valid_stats: List[int] = [200, 206, 302],
                  api_threaded: bool = True,
-                 test_anonymous: bool = True) -> tuple:
+                 test_anonymous: bool = True,
+                 keep_process_bar_style: bool = True,
+                 keep_process_bar_style_file: object = sys.stderr) -> tuple:
     """
+    :param keep_process_bar_style_file: 保持进度条格式时，需要的输出流参数
+    :param keep_process_bar_style: 是否保持进度条格式，如果此项为True，则log函数中参数print_file参数不会生效
     :param test_anonymous: 是否仅获取匿名代理，默认为True
     :param api_threaded:
     :param test_valid_stats: 测试代理时，指定什么状态为代理可以使用
@@ -421,7 +437,8 @@ def basic_config(log_file_name: str = "crawler_util.log",
          (GETTER_TIMEOUT, getter_timeout), (TESTER_URL, tester_url),
          (ENABLE_TESTER, enable_tester), (ENABLE_GETTER, enable_getter),
          (ENABLE_SERVER, enable_server), (TEST_VALID_STATUS, test_valid_stats), (TEST_ANONYMOUS, test_anonymous),
-         (API_THREADED, api_threaded)])
+         (API_THREADED, api_threaded), (KEEP_PROCESS_BAR_STYLE, keep_process_bar_style),
+         (KEEP_PROCESS_BAR_STYLE_FILE, keep_process_bar_style_file)])
     PROXY_POOL_URL = proxy_pool_url if len(proxy_pool_url) != 0 else PROXY_POOL_URL
     log_style = logs_style
     if require_proxy_pool and PROXY_POOL_CAN_RUN_FLAG and len(
@@ -639,7 +656,7 @@ def save_obj(src: object, path: str = "") -> bool:
     :param path: 保存的路径
     :return: 返回是否保存成功,True 表示成功
     """
-    path = path if len(path) > 0 else local_path_generate("", suffix="." + type(src))
+    path = path if len(path) > 0 else local_path_generate("", suffix="." + str(type(src)))
     try:
         with open(path, mode="wb") as f:
             pickle.dump(src, f, pickle.HIGHEST_PROTOCOL)
@@ -699,14 +716,15 @@ class ThreadStopException(Exception):
     def __repr__(self) -> str:
         return super().__repr__()
 
-# if __name__ == "__main__":
-#     a = CanStopThread()
-#     basic_config(logs_style=LOG_STYLE_PRINT)
-#     try:
-#         a.stop()
-#     except ThreadStopException as e:
-#         log("线程{}结束".format(a.name))
-#     except Exception as e:
-#         log("结束线程{}错误{}".format(a.name, e))
-#     except BaseException as e:
-#         log(e)
+if __name__ == "__main__":
+    basic_config(logs_style=LOG_STYLE_PRINT)
+    bar = process_bar()
+    bar.process(0, 1, 100)
+    for i in range(100):
+        bar.process(0, 1, 100)
+        log("ahfu")
+        time.sleep(0.1)
+#     for batch in tqdm(range(100), total=100, position=0, file=sys.stdout, desc="desc"):
+#         if batch % 5 == 0:
+#             tqdm.write(str(batch))
+#             time.sleep(1)
